#
# REFLOW OVEN GUI
#
# Brendon Hackson - btj28
# Joseph Featherston - jgf82
#

import pygame
import os, sys

from sync import *


size = (width,height) = (320,240)
black = (0,0,0)
white = (255,255,255)
color_1 = (200,50,0)
color_2 = (50,100,150)
label_color = white
value_color = color_2
positive_color = (0,100,30)
negative_color = color_1

os.putenv('SDL_VIDEODRIVER', 'fbcon')
os.putenv('SDL_FBDEV', '/dev/fb1')
os.putenv('SDL_MOUSEDRV', 'TSLIB')
os.putenv('SDL_MOUSEDEV', '/dev/input/touchscreen')

pygame.init()

screen = pygame.display.set_mode(size)
screen.fill(black)
pygame.display.flip()


sm_font = pygame.font.Font(None, 20)
md_font = pygame.font.Font(None, 40)
lg_font = pygame.font.Font(None, 80)


def drawStart():
  start_text  = [
    ('Preheat Temp',(60,20),label_color,sm_font),
    (getValue("preheat_temp"),(60,50),value_color,sm_font),
    ('Preheat Time',(60,80),label_color,sm_font),
    (getValue("preheat_time"),(60,110),value_color,sm_font),
    ('Preheat Rate',(60,140),label_color,sm_font),
    (getValue("preheat_rate"),(60,170),value_color,sm_font),
    ('Peak Temp',(180,20),label_color,sm_font),
    (getValue("peak_temp"),(180,50),value_color,sm_font),
    ('Peak Rate',(180,80),label_color,sm_font),
    (getValue("peak_rate"),(180,110),value_color,sm_font),
    ('Cool Rate',(180,140),label_color,sm_font),
    (getValue("cool_rate"),(180,170),value_color,sm_font),
    ('+',(90,45),positive_color,lg_font),
    ('-',(30,45),negative_color,lg_font),
    ('+',(90,105),positive_color,lg_font),
    ('-',(30,105),negative_color,lg_font),
    ('+',(90,165),positive_color,lg_font),
    ('-',(30,165),negative_color,lg_font),
    ('+',(210,45),positive_color,lg_font),
    ('-',(150,45),negative_color,lg_font),
    ('+',(210,105),positive_color,lg_font),
    ('-',(150,105),negative_color,lg_font),
    ('+',(210,165),positive_color,lg_font),
    ('-',(150,165),negative_color,lg_font),
    ]
  screen.fill(black)
  for text,pos,color,font in start_text:
    text = font.render(text, True, color)
    text_rect = text.get_rect(center=pos)
    screen.blit(text,text_rect)
  text = lg_font.render('GO', True, white)
  text_rect = text.get_rect(center=(275,40))
  screen.fill(positive_color,text_rect)
  screen.blit(text,text_rect)
  pygame.display.flip()


def handleStart():
  drawStart()
  events = pygame.event.get()
  for event in events:
    if event.type == pygame.KEYDOWN:
      if event.key == pygame.K_ESCAPE:
        return False
    if event.type ==  pygame.MOUSEBUTTONDOWN:
      x, y = pygame.mouse.get_pos()
      if isPressed(x,y,275,40,20):
        return False
      elif isPressed(x,y,90,45,20):
        incValue("preheat_temp")
      elif isPressed(x,y,30,45,20):
        decValue("preheat_temp")
      elif isPressed(x,y,90,105,20):
        incValue("preheat_time")
      elif isPressed(x,y,30,105,20):
        decValue("preheat_time")
      elif isPressed(x,y,90,165,20):
        incValue("preheat_rate")
      elif isPressed(x,y,30,165,20):
        decValue("preheat_rate")
      elif isPressed(x,y,210,45,20):
        incValue("peak_temp")
      elif isPressed(x,y,150,45,20):
        decValue("peak_temp")
      elif isPressed(x,y,210,105,20):
        incValue("peak_rate")
      elif isPressed(x,y,150,105,20):
        decValue("peak_rate")
      elif isPressed(x,y,210,165,20):
        incValue("cool_rate")
      elif isPressed(x,y,150,165,20):
        decValue("cool_rate")

  return True


def isPressed(x,y,cen_x,cen_y,size):
  if x<cen_x+size and x>cen_x-size:
    if y<cen_y+size and y>cen_y-size:
      return True
  return False

def drawRun():
  run_text  = [
    ('Current Temp',(70,20),label_color,sm_font),
    (getState("temp"),(70,50),value_color,md_font),
    ('Elapsed Time',(70,80),label_color,sm_font),
    (getState("time"),(70,110),value_color,md_font),
    ('Desired Temp',(250,20),label_color,sm_font),
    (getState("desired_temp"),(250,50),value_color,md_font),
    ('Current Stage',(250,80),label_color,sm_font),
    (getState("state"),(250,110),value_color,md_font),
    ]
  screen.fill(black)
  for text,pos,color,font in run_text:
    text = font.render(text, True, color)
    text_rect = text.get_rect(center=pos)
    screen.blit(text,text_rect)
  text = lg_font.render('STOP', True, white)
  text_rect = text.get_rect(center=(160,180))
  screen.fill(negative_color,text_rect)
  screen.blit(text,text_rect)
  pygame.display.flip()





def handleRun():
  drawRun()
  events = pygame.event.get()
  for event in events:
    if event.type == pygame.KEYDOWN:
      if event.key == pygame.K_ESCAPE:
        return False
    if event.type ==  pygame.MOUSEBUTTONDOWN:
      x, y = pygame.mouse.get_pos()
      if isPressed(x,y,160,180,20):
        return False
  return True









